/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "main.h"

// For command line options.
gchar* command_line=NULL;
gchar** parameter=NULL;

// For showing the Menu if right click on a vtebox.
extern gboolean show_menu;

GtkWidget *window;
GtkWidget *notebook;

int main( int   argc,
		  char *argv[] )
{
	bindtextdomain (PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (PACKAGE, "UTF-8");
	textdomain (PACKAGE);

	// deal the command line optins
	command_option(argc, argv);

	// init the gtk+2 engine
	gtk_init(&argc, &argv);

	// deal user settings
	get_user_settings();

	// create main window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), FULLPACKAGENAME " - " VERSION);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	GdkPixbuf *icon = gdk_pixbuf_new_from_file(ICONDIR G_DIR_SEPARATOR_S PACKAGE ".png", NULL);
	gtk_window_set_icon(GTK_WINDOW(window), icon);
	if (icon)
		g_object_unref(icon);

	// close application if [Close Button] clicked
	g_signal_connect(G_OBJECT(window), "delete_event",
					  G_CALLBACK(gtk_main_quit), NULL);

	// create notebook
	notebook = gtk_notebook_new();
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_container_add(GTK_CONTAINER(window), notebook);

	// add a new page to notebook. run_once=TRUE.
	add_page(TRUE);

	// create menu for switching locales
	if (show_menu)
		create_menu();
	// else
	//	g_debug("Sorry, the menu won't be shown!\n");

	// finish!
	gtk_widget_show_all(window);
	gtk_main();

	return 0;
}

void command_option(int   argc,
					char *argv[])
{
	gint i;
	for (i=0;i<argc;i++)
	{
		// g_debug("%2d: %s\n",i, argv[i]);
		if ((!strcmp(argv[i], "-v")) || (!strcmp(argv[i], "--version")))
		{
			g_print("%s %s\n", FULLPACKAGENAME, VERSION);
			exit (0);
		}
		else if ((!strcmp(argv[i], "-h")) || (!strcmp(argv[i], "--help")))
		{
			g_print(_("%s is a libvte based X Terminal Emulater.\n\n"), FULLPACKAGENAME);
			g_print(_("Use -e/--execute to run a command after %s start.\n"), FULLPACKAGENAME);
			g_print(_("Use -v/--version to show the version infomation.\n"));
			g_print(_("Use -p/--profile to got a profile sample.\n"));
			g_print(_("Your %s profile is: %s/%s\n\n"),
					  FULLPACKAGENAME, g_get_user_config_dir(), RCFILE);
			g_print(_("Default shortcut key: (It may custom by editing user's profile)\n\n"
				  "  * <Ctrl><T>\t\tNew tab\n"
				  "  * <Ctrl><W>\t\tClose current tab\n"
				  "  * <Ctrl><E>\t\tRename current tab\n"
				  "  * <Ctrl><PageUp>\tSwitch to prev tab\n"
				  "  * <Ctrl><PageDown>\tSwitch to next tab\n"
				  "  * <Ctrl><Home>\tSwitch to first tab\n"
				  "  * <Ctrl><End>\t\tSwitch to last tab\n"
				  "  * <Ctrl><Left>\tMove current page forward\n"
				  "  * <Ctrl><Right>\tMove current page backward\n"
				  "  * <Ctrl><Up>\t\tMove current page to first\n"
				  "  * <Ctrl><Down>\tMove current page to last\n"
				  "  * <Shift><Insert>\tPast the text in clipboard\n\n"));
			g_print(_("Please report bug to %s.\n"), AUTHOR_EMAIL);
			exit (0);
		}
		else if ((!strcmp(argv[i], "-p")) || (!strcmp(argv[i], "--profile")))
		{
			g_print("[main]\n"
				"page_name = Terminal\n"
				"foreground_color = white\n"
				"background_color = black\n"
				"font_name = Monospace 15\n"
				"screen_with = 80\n"
				"screen_height = 25\n"
				"transparent_background = 0\n"
				"background_saturation = 0.2\n"
				"word_chars = -A-Za-z0-9_$.+!*(),;:@&=?/~#%[]\n"
				"scrollback_lines = 1024\n"
				"locales_list = EUC-JP GB2312 Big5\n"
				"\n"
				"[key]\n"
				"new_tab_key = Ctrl T\n"
				"close_tab_key = Ctrl W\n"
				"edit_label_key = Ctrl E\n"
				"prev_tab_key = Ctrl Page_Up\n"
				"next_tab_key = Ctrl Page_Down\n"
				"first_tab_key = Ctrl Home\n"
				"last_tab_key = Ctrl End\n"
				"move_tab_forward = Ctrl Left\n"
				"move_tab_backward = Ctrl Right\n"
				"move_tab_first = Ctrl Up\n"
				"move_tab_last = Ctrl Down\n");
			exit (0);
		}
		else if ((!strcmp(argv[i], "-e")) || (!strcmp(argv[i], "--execute")))
		{
			if (++i==argc)
				g_debug("Missing command after -e/--execute option!\n");
			else
			{
				command_line = argv[i];
				parameter = &(argv[i]);
				// g_debug("Command = %s %s\n", command_line, *parameter);
			}
		}
	}
}

