/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "notebook.h"

// For command line options.
extern gchar* command_line;
extern gchar** parameter;
// The defalut Page Name
extern gchar *page_name;
// For using transparent background
extern gboolean transparent_background;
extern gdouble background_saturation;

// Don't show menu if there is no locales_list in rc files
extern gboolean show_menu;
// For showing the Menu if right click on a vtebox.
extern GtkWidget *locale_menu;
// For setting the size when adding a new page, avoid to set it to 80x24 (libvte default size)
extern gint screen_with;
extern gint screen_height;
// For sloving the function key
extern struct KeyValue pagekeys[KEYS];
// For increase/decrease the size of font
extern gchar* font_name;
// For Transparent Background
extern GtkWidget *trans_bg_item;

// For Set the radio_list to System Defalut
extern GtkWidget *default_encoding;

extern GtkWidget *window;
extern GtkWidget *notebook;

GtkWidget *current_vtebox=NULL;

void add_page(gboolean run_once)
{
	// the component of a single page
	struct Page *new_page = g_new0(struct Page, 1);
	gint current_page_no;
	const gchar *directory = g_getenv("PWD");

	new_page->label = gtk_label_new(page_name);
	new_page->hbox = gtk_hbox_new(FALSE, 0);

	// create vtebox
	new_page->vtebox = vte_terminal_new();

	// Get current vtebox size and directory. for init a new tab.
	current_page_no = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	if ( current_page_no != -1 && current_vtebox != NULL)
	{
		screen_with = vte_terminal_get_column_count(VTE_TERMINAL(current_vtebox));
		screen_height = vte_terminal_get_row_count(VTE_TERMINAL(current_vtebox));
		// g_debug("Trying to fix vtebox to %d x %d when showing the tab bar\n",
		//	   screen_with, screen_height);

		// Got the current page's directory
		struct Page *prev_page = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
		directory = g_file_read_link(g_strdup_printf("/proc/%d/cwd", prev_page->pid), NULL);
		// g_debug("use the directory %s (/proc/%d/cwd)\n", directory, prev_page->pid);
	}

	// Init new page. run_once: some settings only need run once.
	// run_once only = TRUE when init LilyTerm in main().
	init_new_page(window, new_page->vtebox, run_once);
	gtk_box_pack_start(GTK_BOX(new_page->hbox), new_page->vtebox, TRUE, TRUE, 0);
	// the close page event
	g_signal_connect(G_OBJECT(new_page->vtebox), "child_exited",
					 G_CALLBACK(close_page), NULL);
	// when get focus, update `current_vtebox'
	g_signal_connect(G_OBJECT(new_page->vtebox), "grab-focus",
					 G_CALLBACK(page_grab_focuse), NULL);
	// when vtebox is focuse, the size of vtebox is unchangeable.
	g_signal_connect_after(G_OBJECT(new_page->vtebox), "focus-in-event",
					 G_CALLBACK(page_get_focuse), NULL);
	// when vtebox is focuse, the size of vtebox is changeable.
	g_signal_connect(G_OBJECT(new_page->vtebox), "focus-out-event",
					 G_CALLBACK(page_lost_focuse), NULL);
	// if function key pressed
	g_signal_connect(G_OBJECT(new_page->vtebox), "key-press-event",
					 G_CALLBACK(vtebox_key_press), NULL);

	if (show_menu)
		// show the LOCALE Selection menu
		g_signal_connect(G_OBJECT(new_page->vtebox), "button-press-event",
						 G_CALLBACK(vtebox_button_press), NULL);

	// scrollbar
	new_page->scrollbar = gtk_vscrollbar_new(vte_terminal_get_adjustment(VTE_TERMINAL(new_page->vtebox)));
	gtk_box_pack_start(GTK_BOX(new_page->hbox), new_page->scrollbar, FALSE, FALSE, 0);

	// add the new page to notebook
	current_page_no = gtk_notebook_append_page(GTK_NOTEBOOK(notebook), new_page->hbox, new_page->label);
	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(notebook), new_page->hbox,TRUE);
	gtk_notebook_set_tab_detachable(GTK_NOTEBOOK(notebook), new_page->hbox,FALSE);
	g_object_set(gtk_widget_get_parent(new_page->label), "can-focus", FALSE, NULL);

	// show the page bar if necessary
	if (current_page_no == 1)
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);

	// Execute programs in the vtebox
	//if (parameter != NULL)
	//	g_debug ("Run Command: %s %s\n", command_line, *parameter);
	new_page->pid = vte_terminal_fork_command(VTE_TERMINAL(new_page->vtebox),
									command_line, parameter, NULL,
									directory, TRUE, TRUE, TRUE);
	// We MUST clear command_line and parameter after run -e option.
	command_line = NULL;
	parameter = NULL;

	// for debug only
	// gchar *pid = g_strdup_printf("%d", new_page->pid);
	// gtk_label_set_text(GTK_LABEL(new_page->label), pid);
	// g_free(pid);

	// finish!
	gtk_widget_show_all(new_page->hbox);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page_no);
	gtk_window_set_focus(GTK_WINDOW(window), new_page->vtebox);

	// Store the new page data
	new_page->encoding = default_encoding;
	new_page->transparent_background = transparent_background;
	new_page->background_saturation = background_saturation;
	g_object_set_data(G_OBJECT(new_page->vtebox), "Data", new_page);

	// gtk_widget_show_all(notebook);
}

void close_page (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	gint total_page=gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	gint current_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	
	// g_debug("Total Page (Notebook):%d\n", total_page);

	// free the memory used by this page
	struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
	// g_debug("Deleting Page ID: %d\n", current_data->pid);
	g_free(current_data);

	// if it is the last page, exit application!
	if (total_page == 1)
		gtk_main_quit();
	else
	{
		// remove current page
		// g_debug ("This page is going to be removed!\n");
		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), current_page);

		if ( current_page < total_page-1 )
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page);

		// hide the page bar if necessary
		if (total_page == 2)
		{
			// hide the page bar
			gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
			// Yes, just set it to 1 x 1,
			// and the window will be resized to correct geometry automatically.
			gtk_window_resize(GTK_WINDOW(window), 1, 1);
		}
	}
}

static void page_grab_focuse(GtkWidget *vtebox, gpointer user_data)
{
	// g_debug ("Update current_vtebox!\n");
	current_vtebox = vtebox;
	// if (current_vtebox!=NULL)
	// {
	//  struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
	//  if (current_data!=NULL)
	//	  g_debug("Change to the %d vtebox(%d), and the pid is %d!\n",
	//		   gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)),
	//		   current_vtebox, current_data->pid);
	// }
}

static void page_get_focuse(GtkWidget *vtebox, gpointer user_data)
{
	// g_debug("vtebox get focuse, vtebox can not resize!\n");
	if (current_vtebox!=NULL)
		window_resizable(window, current_vtebox, FALSE, -1);
}

static void page_lost_focuse(GtkWidget *vtebox, gpointer user_data)
{
	// g_debug("vtebox lost focuse, vtebox can be resized!\n");
	if (current_vtebox!=NULL)
		window_resizable(window, current_vtebox, FALSE, 1);
}

gboolean vtebox_button_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	if (event->button == 3)
	{
		struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(current_data->encoding), TRUE);
		GTK_CHECK_MENU_ITEM(trans_bg_item)->active = current_data->transparent_background;
		gtk_menu_popup(GTK_MENU(locale_menu), NULL, NULL, NULL, NULL, event->button, event->time);
		return TRUE;
	}
	return FALSE;
}

static gboolean vtebox_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	// don't check if no ctrl/shift/alt key pressed!
	if (event->state & ALL_ACCELS_MASK)
	{
		// don't check if only shift key pressed!
		if (event->state & SHIFT_ONLY_MASK)
		{
			// g_debug("ALL_ACCELS_MASK = %X\n", ALL_ACCELS_MASK);
			// g_debug("Got the function key: %s (status: %X, check: %X)\n",
			//	   gdk_keyval_name(event->keyval), event->state,
			//	   event->state|GDK_LOCK_MASK|GDK_MOD2_MASK);
			gint i, keyval, mods;
			if ((event->keyval>=GDK_a) && (event->keyval<=GDK_z))
				keyval = event->keyval - GDK_a + GDK_A;
			else
				keyval = event->keyval;
			mods = event->state|GDK_LOCK_MASK|GDK_MOD2_MASK;
			for (i=0;i<KEYS;i++)
			{
				// g_debug("Checking... key: %s (status: %X)\n",
				//	   gdk_keyval_name(pagekeys[i].key), event->state|GDK_LOCK_MASK);
				if ((mods==pagekeys[i].mods) && (keyval==pagekeys[i].key))
				{
					// deal the function key
					deal_key_press(i);
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

// Default function key:
//  [0] <Ctrl><T> = New Page
//  [1] <Ctrl><W> = Close Page
//  [2] <Ctrl><E> = Edit Page Label
//  [3] <Ctrl><V> = Past Page Label
//  [4] <Ctrl><PageUp> = Pre Page
//  [5] <Ctrl><PageDown> = Next Page
//  [6] <Ctrl><Home> = First Page
//  [7] <Ctrl><End> = Last Page
//  [8] <Ctrl><E> = Move Page Forward
//  [9] <Ctrl><C> = Move Page Backward
// [10] <Ctrl><E> = Move Page to First
// [11] <Ctrl><C> = Move Page to Last
//		<Shift><Insert> to pase clipboard.
static void deal_key_press(gint type)
{
	gint current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	gint total_page = gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");

	switch (type)
	{
		case 0:
		{
			// add a new page
			add_page(FALSE);
			break;
		}
		case 1:
		{
			// close page
			close_page (NULL,NULL,NULL);
			break;
		}
		case 2:
		{
			// edit page's label
			edit_page_label_dialog();
			break;
		}
		case 3:
		{
			// past page's label
			g_debug("Trying to rename the label whith the selected text.\n");
			break;
		}
		case 4:
		{
			// switch to pre page
			if (current_page)
				gtk_notebook_prev_page(GTK_NOTEBOOK(notebook));
			else
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), total_page -1);
			break;
		}
		case 5:
		{
			// switch to next page
			if (current_page == (gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1))
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
			else
				gtk_notebook_next_page(GTK_NOTEBOOK(notebook));
			break;
		}
		case 6:
		{
			// switch to first page
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
			break;
		}
		case 7:
		{
			// switch to last page
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), total_page-1);
			break;
		}
		case 8:
		{
			// move current page forward
			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, current_page -1);
			break;
		}
		case 9:
		{
			// move current page backward
			if (current_page == (gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1))
				 gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, 0);
			else
				gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook),
							   current_data->hbox, current_page + 1);
			break;
		}
		case 10:
		{
			// move current page to first
			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, 0);
			break;
		}
		case 11:
		{
			// move current page to last
			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, -1);
			break;
		}
		case 12:
		{
			// Increase the font size
			gint i;
			PangoFontDescription *font_desc = pango_font_description_from_string(font_name);
			gint fontsize = pango_font_description_get_size (font_desc)/PANGO_SCALE*1.15;
			// g_debug("Trying to increase the font size to %d.\n", fontsize);
			pango_font_description_set_size (font_desc, fontsize*PANGO_SCALE);
			g_free(font_name);
			font_name = pango_font_description_to_string (font_desc);
			// g_debug("The final font name = %s\n", font_name);
			break;
		}
	}
}

void edit_page_label_dialog()
{
	GtkWidget *dialog, *main_hbox, *icon_vbox, *icon, *main_right_vbox,
			  *state_vbox, *title_hbox, *title, *entry_hbox, *label, *entry, *state_bottom_hbox;

	//   /--------------main_hbox--------------\
	//   |/icon_vbox\/------state_vbox-----\/-\|
	//   ||		||/----title_hbox-----\||r||
	//   ||		|||		      |||i||
	//   ||		||\-------------------/||g||
	//   ||		||/----entry_hbox-----\||h||
	//   ||		|||		      |||t||
	//   ||		|||		      ||| ||
	//   ||		||\-------------------/||v||
	//   ||		||/-state_bottom_hbox-\||b||
	//   ||		|||		       ||o||
	//   ||		||\-------------------/||x||
	//   |\---------/\---------------------/\-/|
	//   |	   /------------\ /------------\   |
	//   |	   |	OK	| |   CANCEL   |   |
	//   |	   \------------/ \------------/   |
	//   \-------------------------------------/

	/* Create the dialog window */
	dialog = gtk_dialog_new_with_buttons (_("Rename the tab"),
						 GTK_WINDOW(window),
						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
						 GTK_STOCK_OK,
						 GTK_RESPONSE_OK,
						 GTK_STOCK_CANCEL,
						 GTK_RESPONSE_CANCEL,
						 NULL);
	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
	gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
	// set default response when press enter in entry.
	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

	main_hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), main_hbox);
	main_right_vbox = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_end (GTK_BOX(main_hbox), main_right_vbox, FALSE, FALSE, 0);

	// information icon, it should be plase at the top of iconvbox
	icon_vbox = gtk_vbox_new (FALSE, 30);
	gtk_box_pack_start (GTK_BOX(main_hbox), icon_vbox, FALSE, FALSE, 0);
	icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO,
									 GTK_ICON_SIZE_DIALOG);
	gtk_box_pack_start (GTK_BOX(icon_vbox), icon, FALSE, FALSE, 0);

	state_vbox = gtk_vbox_new (FALSE, 15);
	gtk_box_pack_start (GTK_BOX(main_hbox), state_vbox, FALSE, FALSE, 0);

	title_hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX(state_vbox), title_hbox, FALSE, FALSE, 0);
	title = gtk_label_new (_("Please input the Label Name of this tab:"));
	gtk_box_pack_start (GTK_BOX(title_hbox), title, FALSE, FALSE, 0);

	state_bottom_hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_end (GTK_BOX(state_vbox), state_bottom_hbox, FALSE, FALSE, 0);

	entry_hbox = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX(state_vbox), entry_hbox, FALSE, FALSE, 10);

	label = gtk_label_new (_("Label Name:"));
	gtk_box_pack_start (GTK_BOX(entry_hbox), label, FALSE, FALSE, 0);

	// entry area
	entry = gtk_entry_new ();
	gtk_entry_set_text(GTK_ENTRY(entry),
				gtk_label_get_text(GTK_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
					gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook),
						gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))))));
	gtk_box_pack_start (GTK_BOX (entry_hbox), entry, TRUE, TRUE, 0);
	// set activate = TRUE when [enter] pressed in entry
	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);

	// We'll use gtk_dialog_run() to show the whole dialog
	gtk_widget_show_all (main_hbox);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
	{
		struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
		// g_debug("Got label name: %s\n",gtk_entry_get_text(GTK_ENTRY(entry)));
		if (strlen(gtk_entry_get_text(GTK_ENTRY(entry))))
			gtk_label_set_text(GTK_LABEL(current_data->label), gtk_entry_get_text(GTK_ENTRY(entry)));
		else
			gtk_label_set_text(GTK_LABEL(current_data->label), page_name);
	}
	gtk_widget_destroy(dialog);
}
